@@ -1,11 +1,13 @@
+import asyncio
 import inspect
 import logging
+from functools import wraps
 from pathlib import Path
 
 import click
 
 from .. import __version__
-from ..api import AppleMusicApi
+from ..api import AppleMusicApi, ItunesApi
 from ..downloader import (
     AppleMusicBaseDownloader,
     AppleMusicDownloader,
@@ -15,21 +17,26 @@ from ..downloader import (
     CoverFormat,
     DownloadItem,
     DownloadMode,
-    MediaDownloadConfigurationError,
-    MediaFormatNotAvailableError,
-    MediaNotStreamableError,
+    GamdlFormatNotAvailableError,
+    GamdlNotStreamableError,
+    GamdlSyncedLyricsOnlyError,
     RemuxFormatMusicVideo,
     RemuxMode,
 )
 from ..interface import (
+    AppleMusicInterface,
+    AppleMusicMusicVideoInterface,
+    AppleMusicSongInterface,
+    AppleMusicUploadedVideoInterface,
     MusicVideoCodec,
     MusicVideoResolution,
     SongCodec,
     SyncedLyricsFormat,
     UploadedVideoQuality,
 )
+from .config_file import ConfigFile
 from .constants import X_NOT_IN_PATH
-from .utils import Csv, CustomLoggerFormatter, PathPrompt, load_config_file, make_sync
+from .utils import Csv, CustomLoggerFormatter, prompt_path
 
 logger = logging.getLogger(__name__)
 
@@ -42,6 +49,40 @@ uploaded_video_downloader_sig = inspect.signature(
 )
 
 
+def load_config_file(
+    ctx: click.Context,
+    param: click.Parameter,
+    no_config_file: bool,
+) -> click.Context:
+    if no_config_file:
+        return ctx
+
+    config_file = ConfigFile(ctx.params["config_path"])
+    config_file.cleanup_unknown_params(ctx.command.params)
+    config_file.add_params_default_to_config(
+        ctx.command.params,
+    )
+    parsed_params = config_file.parse_params_from_config(
+        [
+            param
+            for param in ctx.command.params
+            if ctx.get_parameter_source(param.name)
+            != click.core.ParameterSource.COMMANDLINE
+        ]
+    )
+    ctx.params.update(parsed_params)
+
+    return ctx
+
+
+def make_sync(func):
+    @wraps(func)
+    def wrapper(*args, **kwargs):
+        return asyncio.run(func(*args, **kwargs))
+
+    return wrapper
+
+
 @click.command()
 @click.help_option("-h", "--help")
 @click.version_option(__version__, "-v", "--version")
@@ -85,7 +126,7 @@ uploaded_video_downloader_sig = inspect.signature(
 @click.option(
     "--cookies-path",
     "-c",
-    type=PathPrompt(is_file=True),
+    type=click.Path(file_okay=True, dir_okay=False, readable=True, resolve_path=True),
     default=api_sig.parameters["cookies_path"].default,
     help="Cookies file path",
 )
@@ -189,6 +230,12 @@ uploaded_video_downloader_sig = inspect.signature(
     default=base_downloader_sig.parameters["compilation_folder_template"].default,
     help="Compilation folder template",
 )
+@click.option(
+    "--no-album-folder-template",
+    type=str,
+    default=base_downloader_sig.parameters["no_album_folder_template"].default,
+    help="No album folder template",
+)
 @click.option(
     "--single-disc-file-template",
     type=str,
@@ -201,12 +248,6 @@ uploaded_video_downloader_sig = inspect.signature(
     default=base_downloader_sig.parameters["multi_disc_file_template"].default,
     help="Multi disc file template",
 )
-@click.option(
-    "--no-album-folder-template",
-    type=str,
-    default=base_downloader_sig.parameters["no_album_folder_template"].default,
-    help="No album folder template",
-)
 @click.option(
     "--no-album-file-template",
     type=str,
@@ -245,7 +286,7 @@ uploaded_video_downloader_sig = inspect.signature(
 )
 # DownloaderSong specific options
 @click.option(
-    "--codec-song",
+    "--song-codec",
     type=SongCodec,
     default=song_downloader_sig.parameters["codec"].default,
     help="Song codec",
@@ -327,16 +368,16 @@ async def main(
     cover_format: CoverFormat,
     album_folder_template: str,
     compilation_folder_template: str,
+    no_album_folder_template: str,
     single_disc_file_template: str,
     multi_disc_file_template: str,
-    no_album_folder_template: str,
     no_album_file_template: str,
     playlist_file_template: str,
     date_tag_template: str,
     exclude_tags: list[str],
     cover_size: int,
     truncate: int,
-    codec_song: SongCodec,
+    song_codec: SongCodec,
     synced_lyrics_format: SyncedLyricsFormat,
     no_synced_lyrics: bool,
     synced_lyrics_only: bool,
@@ -360,28 +401,43 @@ async def main(
         file_handler.setFormatter(CustomLoggerFormatter(use_colors=False))
         root_logger.addHandler(file_handler)
 
+    cookies_path = prompt_path(cookies_path)
+
     logger.info(f"Starting Gamdl {__version__}")
 
-    api = AppleMusicApi.from_netscape_cookies(
+    apple_music_api = AppleMusicApi.from_netscape_cookies(
         cookies_path=cookies_path,
         language=language,
     )
-    await api.setup()
+    await apple_music_api.setup()
 
-    if not api.account_info["meta"]["subscription"]["active"]:
+    itunes_api = ItunesApi(
+        apple_music_api.storefront,
+        apple_music_api.language,
+    )
+    itunes_api.setup()
+
+    if not apple_music_api.account_info["meta"]["subscription"]["active"]:
         logger.critical(
             "No active Apple Music subscription found, you won't be able to download"
             " anything"
         )
         return
-    if api.account_info["data"][0]["attributes"].get("restrictions"):
+    if apple_music_api.account_info["data"][0]["attributes"].get("restrictions"):
         logger.warning(
             "Your account has content restrictions enabled, some content may not be"
             " downloadable"
         )
 
+    interface = AppleMusicInterface(
+        apple_music_api,
+        itunes_api,
+    )
+    song_interface = AppleMusicSongInterface(interface)
+    music_video_interface = AppleMusicMusicVideoInterface(interface)
+    uploaded_video_interface = AppleMusicUploadedVideoInterface(interface)
+
     base_downloader = AppleMusicBaseDownloader(
-        apple_music_api=api,
         output_path=output_path,
         temp_path=temp_path,
         wvd_path=wvd_path,
@@ -397,9 +453,9 @@ async def main(
         cover_format=cover_format,
         album_folder_template=album_folder_template,
         compilation_folder_template=compilation_folder_template,
+        no_album_folder_template=no_album_folder_template,
         single_disc_file_template=single_disc_file_template,
         multi_disc_file_template=multi_disc_file_template,
-        no_album_folder_template=no_album_folder_template,
         no_album_file_template=no_album_file_template,
         playlist_file_template=playlist_file_template,
         date_tag_template=date_tag_template,
@@ -408,35 +464,32 @@ async def main(
         truncate=truncate,
     )
     base_downloader.setup()
-
     song_downloader = AppleMusicSongDownloader(
-        base_downloader,
-        codec=codec_song,
+        base_downloader=base_downloader,
+        interface=song_interface,
+        codec=song_codec,
         synced_lyrics_format=synced_lyrics_format,
         no_synced_lyrics=no_synced_lyrics,
         synced_lyrics_only=synced_lyrics_only,
     )
-    song_downloader.setup()
-
     music_video_downloader = AppleMusicMusicVideoDownloader(
-        base_downloader,
+        base_downloader=base_downloader,
+        interface=music_video_interface,
         codec_priority=music_video_codec_priority,
         remux_format=music_video_remux_format,
         resolution=music_video_resolution,
     )
-    music_video_downloader.setup()
-
     uploaded_video_downloader = AppleMusicUploadedVideoDownloader(
-        base_downloader,
+        base_downloader=base_downloader,
+        interface=uploaded_video_interface,
         quality=uploaded_video_quality,
     )
-    uploaded_video_downloader.setup()
-
     downloader = AppleMusicDownloader(
-        base_downloader,
-        song_downloader,
-        music_video_downloader,
-        uploaded_video_downloader,
+        interface=interface,
+        base_downloader=base_downloader,
+        song_downloader=song_downloader,
+        music_video_downloader=music_video_downloader,
+        uploaded_video_downloader=uploaded_video_downloader,
     )
 
     if not synced_lyrics_only:
@@ -452,7 +505,7 @@ async def main(
 
         if (
             not base_downloader.full_mp4decrypt_path
-            and codec_song
+            and song_codec
             not in (
                 SongCodec.AAC_LEGACY,
                 SongCodec.AAC_HE_LEGACY,
@@ -479,7 +532,7 @@ async def main(
             )
             downloader.skip_music_videos = True
 
-        if not codec_song.is_legacy():
+        if not song_codec.is_legacy():
             logger.warning(
                 "You have chosen an experimental song codec. "
                 "They're not guaranteed to work due to API limitations."
@@ -552,9 +605,9 @@ async def main(
                 await downloader.download(download_item)
             except (
                 FileExistsError,
-                MediaNotStreamableError,
-                MediaFormatNotAvailableError,
-                MediaDownloadConfigurationError,
+                GamdlNotStreamableError,
+                GamdlFormatNotAvailableError,
+                GamdlSyncedLyricsOnlyError,
             ) as e:
                 logger.warning(
                     download_queue_progress + f' Skipping "{media_title}": {e}'
