@@ -1,4 +1,5 @@
 from dataclasses import dataclass
+from typing import Any
 
 from ..interface.types import (
     DecryptionKeyAv,
@@ -12,6 +13,7 @@ from ..interface.types import (
 @dataclass
 class DownloadItem:
     media_metadata: dict = None
+    playlist_metadata: dict = None
     random_uuid: str = None
     lyrics: Lyrics = None
     media_tags: MediaTags = None
@@ -24,6 +26,8 @@ class DownloadItem:
     playlist_file_path: str = None
     synced_lyrics_path: str = None
     cover_path: str = None
+    flat_filter_result: Any = None
+    error: Exception = None
 
 
 @dataclass
